# color codes obtained from https://www.webucator.com/blog/2015/03/python-color-constants-module/

class Color(object):
    """
    Represents a color within bluedot. Used to change the color of the dot.

    Color objects are immutable.

    :param int red:
        The red value of the color `0 - 255`. Default is `255`.

    :param int green:
        The green value of the color `0 - 255`. Default is `255`.

    :param int green:
        The blue value of the color `0 - 255`. Default is `255`.

    :param int green:
        The alpha value of the color `0 - 255`. `0` is transparent. Default 
        is `255`.
    """
    def __init__(self, red = 255, green = 255, blue = 255, alpha = 255):
        self._red = red
        self._green = green
        self._blue = blue
        self._alpha = alpha

    @property
    def red(self):
        """
        Returns the red value of the color.
        """
        return self._red

    @property
    def green(self):
        """
        Returns the green value of the color.
        """
        return self._green

    @property
    def blue(self):
        """
        Returns the blue value of the color.
        """
        return self._blue

    @property
    def alpha(self):
        """
        Returns the alpha value of the color.
        """
        return self._alpha

    @property
    def rgb(self):
        """
        Returns a tuple of `(red, green, blue)` values.
        """
        return (self._red, self._green, self._blue)

    @property
    def rgba(self):
        """
        Returns a tuple of `(red, green, blue, alpha)` values.
        """
        return (self._red, self._green, self._blue, self._alpha)

    @property
    def str_rgb(self):
        """
        Returns a string of red, green, blue hex values in the format
        `#rrggbb`.
        """
        return '#%02x%02x%02x' % (self._red, self._green, self._blue)

    @property
    def str_rgba(self):
        """
        Returns a string of red, green, blue, alpha hex values in the format
        `#rrggbbaa`.
        """
        return '#%02x%02x%02x%02x' % (self._red, self._green, self._blue, self._alpha)

    @property
    def str_argb(self):
        """
        Returns a string of alpha, red, green, blue hex values in the format
        `#aarrggbb`.
        """
        return '#%02x%02x%02x%02x' % (self._alpha, self._red, self._green, self._blue)

    def get_adjusted_color(self, factor):
        """
        Returns a new Color object based on this Color adjusted by a factor

        :param float factor:
            The value to adjust this color by. 
        """
        return Color(self.red * factor, self.green * factor, self.blue * factor)

    def __eq__(self, other):
        other = parse_color(other)
        return self._red == other._red and self._green == other._green and self._blue == other._blue and self._alpha == other._alpha

    def __str__(self):
        return self.str_rgba

ALICEBLUE = Color(240, 248, 255)
ANTIQUEWHITE = Color(250, 235, 215)
ANTIQUEWHITE1 = Color(255, 239, 219)
ANTIQUEWHITE2 = Color(238, 223, 204)
ANTIQUEWHITE3 = Color(205, 192, 176)
ANTIQUEWHITE4 = Color(139, 131, 120)
AQUA = Color(0, 255, 255)
AQUAMARINE1 = Color(127, 255, 212)
AQUAMARINE2 = Color(118, 238, 198)
AQUAMARINE3 = Color(102, 205, 170)
AQUAMARINE4 = Color(69, 139, 116)
AZURE1 = Color(240, 255, 255)
AZURE2 = Color(224, 238, 238)
AZURE3 = Color(193, 205, 205)
AZURE4 = Color(131, 139, 139)
BANANA = Color(227, 207, 87)
BEIGE = Color(245, 245, 220)
BISQUE1 = Color(255, 228, 196)
BISQUE2 = Color(238, 213, 183)
BISQUE3 = Color(205, 183, 158)
BISQUE4 = Color(139, 125, 107)
BLACK = Color(0, 0, 0)
BLANCHEDALMOND = Color(255, 235, 205)
BLUE = Color(0, 0, 255)
BLUE2 = Color(0, 0, 238)
BLUE3 = Color(0, 0, 205)
BLUE4 = Color(0, 0, 139)
BLUEVIOLET = Color(138, 43, 226)
BRICK = Color(156, 102, 31)
BROWN = Color(165, 42, 42)
BROWN1 = Color(255, 64, 64)
BROWN2 = Color(238, 59, 59)
BROWN3 = Color(205, 51, 51)
BROWN4 = Color(139, 35, 35)
BURLYWOOD = Color(222, 184, 135)
BURLYWOOD1 = Color(255, 211, 155)
BURLYWOOD2 = Color(238, 197, 145)
BURLYWOOD3 = Color(205, 170, 125)
BURLYWOOD4 = Color(139, 115, 85)
BURNTSIENNA = Color(138, 54, 15)
BURNTUMBER = Color(138, 51, 36)
CADETBLUE = Color(95, 158, 160)
CADETBLUE1 = Color(152, 245, 255)
CADETBLUE2 = Color(142, 229, 238)
CADETBLUE3 = Color(122, 197, 205)
CADETBLUE4 = Color(83, 134, 139)
CADMIUMORANGE = Color(255, 97, 3)
CADMIUMYELLOW = Color(255, 153, 18)
CARROT = Color(237, 145, 33)
CHARTREUSE1 = Color(127, 255, 0)
CHARTREUSE2 = Color(118, 238, 0)
CHARTREUSE3 = Color(102, 205, 0)
CHARTREUSE4 = Color(69, 139, 0)
CHOCOLATE = Color(210, 105, 30)
CHOCOLATE1 = Color(255, 127, 36)
CHOCOLATE2 = Color(238, 118, 33)
CHOCOLATE3 = Color(205, 102, 29)
CHOCOLATE4 = Color(139, 69, 19)
COBALT = Color(61, 89, 171)
COBALTGREEN = Color(61, 145, 64)
COLDGREY = Color(128, 138, 135)
CORAL = Color(255, 127, 80)
CORAL1 = Color(255, 114, 86)
CORAL2 = Color(238, 106, 80)
CORAL3 = Color(205, 91, 69)
CORAL4 = Color(139, 62, 47)
CORNFLOWERBLUE = Color(100, 149, 237)
CORNSILK1 = Color(255, 248, 220)
CORNSILK2 = Color(238, 232, 205)
CORNSILK3 = Color(205, 200, 177)
CORNSILK4 = Color(139, 136, 120)
CRIMSON = Color(220, 20, 60)
CYAN1 = Color(0, 238, 238)
CYAN2 = Color(0, 205, 205)
CYAN3 = Color(0, 139, 139)
DARKGOLDENROD = Color(184, 134, 11)
DARKGOLDENROD1 = Color(255, 185, 15)
DARKGOLDENROD2 = Color(238, 173, 14)
DARKGOLDENROD3 = Color(205, 149, 12)
DARKGOLDENROD4 = Color(139, 101, 8)
DARKGRAY = Color(169, 169, 169)
DARKGREEN = Color(0, 100, 0)
DARKKHAKI = Color(189, 183, 107)
DARKOLIVEGREEN = Color(85, 107, 47)
DARKOLIVEGREEN1 = Color(202, 255, 112)
DARKOLIVEGREEN2 = Color(188, 238, 104)
DARKOLIVEGREEN3 = Color(162, 205, 90)
DARKOLIVEGREEN4 = Color(110, 139, 61)
DARKORANGE = Color(255, 140, 0)
DARKORANGE1 = Color(255, 127, 0)
DARKORANGE2 = Color(238, 118, 0)
DARKORANGE3 = Color(205, 102, 0)
DARKORANGE4 = Color(139, 69, 0)
DARKORCHID = Color(153, 50, 204)
DARKORCHID1 = Color(191, 62, 255)
DARKORCHID2 = Color(178, 58, 238)
DARKORCHID3 = Color(154, 50, 205)
DARKORCHID4 = Color(104, 34, 139)
DARKSALMON = Color(233, 150, 122)
DARKSEAGREEN = Color(143, 188, 143)
DARKSEAGREEN1 = Color(193, 255, 193)
DARKSEAGREEN2 = Color(180, 238, 180)
DARKSEAGREEN3 = Color(155, 205, 155)
DARKSEAGREEN4 = Color(105, 139, 105)
DARKSLATEBLUE = Color(72, 61, 139)
DARKSLATEGRAY = Color(47, 79, 79)
DARKSLATEGRAY1 = Color(151, 255, 255)
DARKSLATEGRAY2 = Color(141, 238, 238)
DARKSLATEGRAY3 = Color(121, 205, 205)
DARKSLATEGRAY4 = Color(82, 139, 139)
DARKTURQUOISE = Color(0, 206, 209)
DARKVIOLET = Color(148, 0, 211)
DEEPPINK1 = Color(255, 20, 147)
DEEPPINK2 = Color(238, 18, 137)
DEEPPINK3 = Color(205, 16, 118)
DEEPPINK4 = Color(139, 10, 80)
DEEPSKYBLUE1 = Color(0, 191, 255)
DEEPSKYBLUE2 = Color(0, 178, 238)
DEEPSKYBLUE3 = Color(0, 154, 205)
DEEPSKYBLUE4 = Color(0, 104, 139)
DIMGRAY = Color(105, 105, 105)
DODGERBLUE1 = Color(30, 144, 255)
DODGERBLUE2 = Color(28, 134, 238)
DODGERBLUE3 = Color(24, 116, 205)
DODGERBLUE4 = Color(16, 78, 139)
EGGSHELL = Color(252, 230, 201)
EMERALDGREEN = Color(0, 201, 87)
FIREBRICK = Color(178, 34, 34)
FIREBRICK1 = Color(255, 48, 48)
FIREBRICK2 = Color(238, 44, 44)
FIREBRICK3 = Color(205, 38, 38)
FIREBRICK4 = Color(139, 26, 26)
FLESH = Color(255, 125, 64)
FLORALWHITE = Color(255, 250, 240)
FORESTGREEN = Color(34, 139, 34)
GAINSBORO = Color(220, 220, 220)
GHOSTWHITE = Color(248, 248, 255)
GOLD1 = Color(255, 215, 0)
GOLD2 = Color(238, 201, 0)
GOLD3 = Color(205, 173, 0)
GOLD4 = Color(139, 117, 0)
GOLDENROD = Color(218, 165, 32)
GOLDENROD1 = Color(255, 193, 37)
GOLDENROD2 = Color(238, 180, 34)
GOLDENROD3 = Color(205, 155, 29)
GOLDENROD4 = Color(139, 105, 20)
GRAY = Color(128, 128, 128)
GRAY1 = Color(3, 3, 3)
GRAY10 = Color(26, 26, 26)
GRAY11 = Color(28, 28, 28)
GRAY12 = Color(31, 31, 31)
GRAY13 = Color(33, 33, 33)
GRAY14 = Color(36, 36, 36)
GRAY15 = Color(38, 38, 38)
GRAY16 = Color(41, 41, 41)
GRAY17 = Color(43, 43, 43)
GRAY18 = Color(46, 46, 46)
GRAY19 = Color(48, 48, 48)
GRAY2 = Color(5, 5, 5)
GRAY20 = Color(51, 51, 51)
GRAY21 = Color(54, 54, 54)
GRAY22 = Color(56, 56, 56)
GRAY23 = Color(59, 59, 59)
GRAY24 = Color(61, 61, 61)
GRAY25 = Color(64, 64, 64)
GRAY26 = Color(66, 66, 66)
GRAY27 = Color(69, 69, 69)
GRAY28 = Color(71, 71, 71)
GRAY29 = Color(74, 74, 74)
GRAY3 = Color(8, 8, 8)
GRAY30 = Color(77, 77, 77)
GRAY31 = Color(79, 79, 79)
GRAY32 = Color(82, 82, 82)
GRAY33 = Color(84, 84, 84)
GRAY34 = Color(87, 87, 87)
GRAY35 = Color(89, 89, 89)
GRAY36 = Color(92, 92, 92)
GRAY37 = Color(94, 94, 94)
GRAY38 = Color(97, 97, 97)
GRAY39 = Color(99, 99, 99)
GRAY4 = Color(10, 10, 10)
GRAY40 = Color(102, 102, 102)
GRAY42 = Color(107, 107, 107)
GRAY43 = Color(110, 110, 110)
GRAY44 = Color(112, 112, 112)
GRAY45 = Color(115, 115, 115)
GRAY46 = Color(117, 117, 117)
GRAY47 = Color(120, 120, 120)
GRAY48 = Color(122, 122, 122)
GRAY49 = Color(125, 125, 125)
GRAY5 = Color(13, 13, 13)
GRAY50 = Color(127, 127, 127)
GRAY51 = Color(130, 130, 130)
GRAY52 = Color(133, 133, 133)
GRAY53 = Color(135, 135, 135)
GRAY54 = Color(138, 138, 138)
GRAY55 = Color(140, 140, 140)
GRAY56 = Color(143, 143, 143)
GRAY57 = Color(145, 145, 145)
GRAY58 = Color(148, 148, 148)
GRAY59 = Color(150, 150, 150)
GRAY6 = Color(15, 15, 15)
GRAY60 = Color(153, 153, 153)
GRAY61 = Color(156, 156, 156)
GRAY62 = Color(158, 158, 158)
GRAY63 = Color(161, 161, 161)
GRAY64 = Color(163, 163, 163)
GRAY65 = Color(166, 166, 166)
GRAY66 = Color(168, 168, 168)
GRAY67 = Color(171, 171, 171)
GRAY68 = Color(173, 173, 173)
GRAY69 = Color(176, 176, 176)
GRAY7 = Color(18, 18, 18)
GRAY70 = Color(179, 179, 179)
GRAY71 = Color(181, 181, 181)
GRAY72 = Color(184, 184, 184)
GRAY73 = Color(186, 186, 186)
GRAY74 = Color(189, 189, 189)
GRAY75 = Color(191, 191, 191)
GRAY76 = Color(194, 194, 194)
GRAY77 = Color(196, 196, 196)
GRAY78 = Color(199, 199, 199)
GRAY79 = Color(201, 201, 201)
GRAY8 = Color(20, 20, 20)
GRAY80 = Color(204, 204, 204)
GRAY81 = Color(207, 207, 207)
GRAY82 = Color(209, 209, 209)
GRAY83 = Color(212, 212, 212)
GRAY84 = Color(214, 214, 214)
GRAY85 = Color(217, 217, 217)
GRAY86 = Color(219, 219, 219)
GRAY87 = Color(222, 222, 222)
GRAY88 = Color(224, 224, 224)
GRAY89 = Color(227, 227, 227)
GRAY9 = Color(23, 23, 23)
GRAY90 = Color(229, 229, 229)
GRAY91 = Color(232, 232, 232)
GRAY92 = Color(235, 235, 235)
GRAY93 = Color(237, 237, 237)
GRAY94 = Color(240, 240, 240)
GRAY95 = Color(242, 242, 242)
GRAY97 = Color(247, 247, 247)
GRAY98 = Color(250, 250, 250)
GRAY99 = Color(252, 252, 252)
GREEN = Color(0, 128, 0)
GREEN1 = Color(0, 255, 0)
GREEN2 = Color(0, 238, 0)
GREEN3 = Color(0, 205, 0)
GREEN4 = Color(0, 139, 0)
GREENYELLOW = Color(173, 255, 47)
HONEYDEW1 = Color(240, 255, 240)
HONEYDEW2 = Color(224, 238, 224)
HONEYDEW3 = Color(193, 205, 193)
HONEYDEW4 = Color(131, 139, 131)
HOTPINK = Color(255, 105, 180)
HOTPINK1 = Color(255, 110, 180)
HOTPINK2 = Color(238, 106, 167)
HOTPINK3 = Color(205, 96, 144)
HOTPINK4 = Color(139, 58, 98)
INDIANRED = Color(176, 23, 31)
INDIANRED = Color(205, 92, 92)
INDIANRED1 = Color(255, 106, 106)
INDIANRED2 = Color(238, 99, 99)
INDIANRED3 = Color(205, 85, 85)
INDIANRED4 = Color(139, 58, 58)
INDIGO = Color(75, 0, 130)
IVORY1 = Color(255, 255, 240)
IVORY2 = Color(238, 238, 224)
IVORY3 = Color(205, 205, 193)
IVORY4 = Color(139, 139, 131)
IVORYBLACK = Color(41, 36, 33)
KHAKI = Color(240, 230, 140)
KHAKI1 = Color(255, 246, 143)
KHAKI2 = Color(238, 230, 133)
KHAKI3 = Color(205, 198, 115)
KHAKI4 = Color(139, 134, 78)
LAVENDER = Color(230, 230, 250)
LAVENDERBLUSH1 = Color(255, 240, 245)
LAVENDERBLUSH2 = Color(238, 224, 229)
LAVENDERBLUSH3 = Color(205, 193, 197)
LAVENDERBLUSH4 = Color(139, 131, 134)
LAWNGREEN = Color(124, 252, 0)
LEMONCHIFFON1 = Color(255, 250, 205)
LEMONCHIFFON2 = Color(238, 233, 191)
LEMONCHIFFON3 = Color(205, 201, 165)
LEMONCHIFFON4 = Color(139, 137, 112)
LIGHTBLUE = Color(173, 216, 230)
LIGHTBLUE1 = Color(191, 239, 255)
LIGHTBLUE2 = Color(178, 223, 238)
LIGHTBLUE3 = Color(154, 192, 205)
LIGHTBLUE4 = Color(104, 131, 139)
LIGHTCORAL = Color(240, 128, 128)
LIGHTCYAN1 = Color(224, 255, 255)
LIGHTCYAN2 = Color(209, 238, 238)
LIGHTCYAN3 = Color(180, 205, 205)
LIGHTCYAN4 = Color(122, 139, 139)
LIGHTGOLDENROD1 = Color(255, 236, 139)
LIGHTGOLDENROD2 = Color(238, 220, 130)
LIGHTGOLDENROD3 = Color(205, 190, 112)
LIGHTGOLDENROD4 = Color(139, 129, 76)
LIGHTGOLDENRODYELLOW = Color(250, 250, 210)
LIGHTGREY = Color(211, 211, 211)
LIGHTPINK = Color(255, 182, 193)
LIGHTPINK1 = Color(255, 174, 185)
LIGHTPINK2 = Color(238, 162, 173)
LIGHTPINK3 = Color(205, 140, 149)
LIGHTPINK4 = Color(139, 95, 101)
LIGHTSALMON1 = Color(255, 160, 122)
LIGHTSALMON2 = Color(238, 149, 114)
LIGHTSALMON3 = Color(205, 129, 98)
LIGHTSALMON4 = Color(139, 87, 66)
LIGHTSEAGREEN = Color(32, 178, 170)
LIGHTSKYBLUE = Color(135, 206, 250)
LIGHTSKYBLUE1 = Color(176, 226, 255)
LIGHTSKYBLUE2 = Color(164, 211, 238)
LIGHTSKYBLUE3 = Color(141, 182, 205)
LIGHTSKYBLUE4 = Color(96, 123, 139)
LIGHTSLATEBLUE = Color(132, 112, 255)
LIGHTSLATEGRAY = Color(119, 136, 153)
LIGHTSTEELBLUE = Color(176, 196, 222)
LIGHTSTEELBLUE1 = Color(202, 225, 255)
LIGHTSTEELBLUE2 = Color(188, 210, 238)
LIGHTSTEELBLUE3 = Color(162, 181, 205)
LIGHTSTEELBLUE4 = Color(110, 123, 139)
LIGHTYELLOW1 = Color(255, 255, 224)
LIGHTYELLOW2 = Color(238, 238, 209)
LIGHTYELLOW3 = Color(205, 205, 180)
LIGHTYELLOW4 = Color(139, 139, 122)
LIMEGREEN = Color(50, 205, 50)
LINEN = Color(250, 240, 230)
MAGENTA = Color(255, 0, 255)
MAGENTA2 = Color(238, 0, 238)
MAGENTA3 = Color(205, 0, 205)
MAGENTA4 = Color(139, 0, 139)
MANGANESEBLUE = Color(3, 168, 158)
MAROON = Color(128, 0, 0)
MAROON1 = Color(255, 52, 179)
MAROON2 = Color(238, 48, 167)
MAROON3 = Color(205, 41, 144)
MAROON4 = Color(139, 28, 98)
MEDIUMORCHID = Color(186, 85, 211)
MEDIUMORCHID1 = Color(224, 102, 255)
MEDIUMORCHID2 = Color(209, 95, 238)
MEDIUMORCHID3 = Color(180, 82, 205)
MEDIUMORCHID4 = Color(122, 55, 139)
MEDIUMPURPLE = Color(147, 112, 219)
MEDIUMPURPLE1 = Color(171, 130, 255)
MEDIUMPURPLE2 = Color(159, 121, 238)
MEDIUMPURPLE3 = Color(137, 104, 205)
MEDIUMPURPLE4 = Color(93, 71, 139)
MEDIUMSEAGREEN = Color(60, 179, 113)
MEDIUMSLATEBLUE = Color(123, 104, 238)
MEDIUMSPRINGGREEN = Color(0, 250, 154)
MEDIUMTURQUOISE = Color(72, 209, 204)
MEDIUMVIOLETRED = Color(199, 21, 133)
MELON = Color(227, 168, 105)
MIDNIGHTBLUE = Color(25, 25, 112)
MINT = Color(189, 252, 201)
MINTCREAM = Color(245, 255, 250)
MISTYROSE1 = Color(255, 228, 225)
MISTYROSE2 = Color(238, 213, 210)
MISTYROSE3 = Color(205, 183, 181)
MISTYROSE4 = Color(139, 125, 123)
MOCCASIN = Color(255, 228, 181)
NAVAJOWHITE1 = Color(255, 222, 173)
NAVAJOWHITE2 = Color(238, 207, 161)
NAVAJOWHITE3 = Color(205, 179, 139)
NAVAJOWHITE4 = Color(139, 121, 94)
NAVY = Color(0, 0, 128)
OLDLACE = Color(253, 245, 230)
OLIVE = Color(128, 128, 0)
OLIVEDRAB = Color(107, 142, 35)
OLIVEDRAB1 = Color(192, 255, 62)
OLIVEDRAB2 = Color(179, 238, 58)
OLIVEDRAB3 = Color(154, 205, 50)
OLIVEDRAB4 = Color(105, 139, 34)
ORANGE = Color(255, 128, 0)
ORANGE1 = Color(255, 165, 0)
ORANGE2 = Color(238, 154, 0)
ORANGE3 = Color(205, 133, 0)
ORANGE4 = Color(139, 90, 0)
ORANGERED1 = Color(255, 69, 0)
ORANGERED2 = Color(238, 64, 0)
ORANGERED3 = Color(205, 55, 0)
ORANGERED4 = Color(139, 37, 0)
ORCHID = Color(218, 112, 214)
ORCHID1 = Color(255, 131, 250)
ORCHID2 = Color(238, 122, 233)
ORCHID3 = Color(205, 105, 201)
ORCHID4 = Color(139, 71, 137)
PALEGOLDENROD = Color(238, 232, 170)
PALEGREEN = Color(152, 251, 152)
PALEGREEN1 = Color(154, 255, 154)
PALEGREEN2 = Color(144, 238, 144)
PALEGREEN3 = Color(124, 205, 124)
PALEGREEN4 = Color(84, 139, 84)
PALETURQUOISE1 = Color(187, 255, 255)
PALETURQUOISE2 = Color(174, 238, 238)
PALETURQUOISE3 = Color(150, 205, 205)
PALETURQUOISE4 = Color(102, 139, 139)
PALEVIOLETRED = Color(219, 112, 147)
PALEVIOLETRED1 = Color(255, 130, 171)
PALEVIOLETRED2 = Color(238, 121, 159)
PALEVIOLETRED3 = Color(205, 104, 137)
PALEVIOLETRED4 = Color(139, 71, 93)
PAPAYAWHIP = Color(255, 239, 213)
PEACHPUFF1 = Color(255, 218, 185)
PEACHPUFF2 = Color(238, 203, 173)
PEACHPUFF3 = Color(205, 175, 149)
PEACHPUFF4 = Color(139, 119, 101)
PEACOCK = Color(51, 161, 201)
PINK = Color(255, 192, 203)
PINK1 = Color(255, 181, 197)
PINK2 = Color(238, 169, 184)
PINK3 = Color(205, 145, 158)
PINK4 = Color(139, 99, 108)
PLUM = Color(221, 160, 221)
PLUM1 = Color(255, 187, 255)
PLUM2 = Color(238, 174, 238)
PLUM3 = Color(205, 150, 205)
PLUM4 = Color(139, 102, 139)
POWDERBLUE = Color(176, 224, 230)
PURPLE = Color(128, 0, 128)
PURPLE1 = Color(155, 48, 255)
PURPLE2 = Color(145, 44, 238)
PURPLE3 = Color(125, 38, 205)
PURPLE4 = Color(85, 26, 139)
RASPBERRY = Color(135, 38, 87)
RAWSIENNA = Color(199, 97, 20)
RED = Color(255, 0, 0)
RED1 = Color(238, 0, 0)
RED2 = Color(205, 0, 0)
RED3 = Color(139, 0, 0)
RED4 = Color(125, 0, 0)
ROSYBROWN = Color(188, 143, 143)
ROSYBROWN1 = Color(255, 193, 193)
ROSYBROWN2 = Color(238, 180, 180)
ROSYBROWN3 = Color(205, 155, 155)
ROSYBROWN4 = Color(139, 105, 105)
ROYALBLUE = Color(65, 105, 225)
ROYALBLUE1 = Color(72, 118, 255)
ROYALBLUE2 = Color(67, 110, 238)
ROYALBLUE3 = Color(58, 95, 205)
ROYALBLUE4 = Color(39, 64, 139)
SALMON = Color(250, 128, 114)
SALMON1 = Color(255, 140, 105)
SALMON2 = Color(238, 130, 98)
SALMON3 = Color(205, 112, 84)
SALMON4 = Color(139, 76, 57)
SANDYBROWN = Color(244, 164, 96)
SAPGREEN = Color(48, 128, 20)
SEAGREEN1 = Color(84, 255, 159)
SEAGREEN2 = Color(78, 238, 148)
SEAGREEN3 = Color(67, 205, 128)
SEAGREEN4 = Color(46, 139, 87)
SEASHELL1 = Color(255, 245, 238)
SEASHELL2 = Color(238, 229, 222)
SEASHELL3 = Color(205, 197, 191)
SEASHELL4 = Color(139, 134, 130)
SEPIA = Color(94, 38, 18)
SGIBEET = Color(142, 56, 142)
SGIBRIGHTGRAY = Color(197, 193, 170)
SGICHARTREUSE = Color(113, 198, 113)
SGIDARKGRAY = Color(85, 85, 85)
SGIGRAY12 = Color(30, 30, 30)
SGIGRAY16 = Color(40, 40, 40)
SGIGRAY32 = Color(81, 81, 81)
SGIGRAY36 = Color(91, 91, 91)
SGIGRAY52 = Color(132, 132, 132)
SGIGRAY56 = Color(142, 142, 142)
SGIGRAY72 = Color(183, 183, 183)
SGIGRAY76 = Color(193, 193, 193)
SGIGRAY92 = Color(234, 234, 234)
SGIGRAY96 = Color(244, 244, 244)
SGILIGHTBLUE = Color(125, 158, 192)
SGILIGHTGRAY = Color(170, 170, 170)
SGIOLIVEDRAB = Color(142, 142, 56)
SGISALMON = Color(198, 113, 113)
SGISLATEBLUE = Color(113, 113, 198)
SGITEAL = Color(56, 142, 142)
SIENNA = Color(160, 82, 45)
SIENNA1 = Color(255, 130, 71)
SIENNA2 = Color(238, 121, 66)
SIENNA3 = Color(205, 104, 57)
SIENNA4 = Color(139, 71, 38)
SILVER = Color(192, 192, 192)
SKYBLUE = Color(135, 206, 235)
SKYBLUE1 = Color(135, 206, 255)
SKYBLUE2 = Color(126, 192, 238)
SKYBLUE3 = Color(108, 166, 205)
SKYBLUE4 = Color(74, 112, 139)
SLATEBLUE = Color(106, 90, 205)
SLATEBLUE1 = Color(131, 111, 255)
SLATEBLUE2 = Color(122, 103, 238)
SLATEBLUE3 = Color(105, 89, 205)
SLATEBLUE4 = Color(71, 60, 139)
SLATEGRAY = Color(112, 128, 144)
SLATEGRAY1 = Color(198, 226, 255)
SLATEGRAY2 = Color(185, 211, 238)
SLATEGRAY3 = Color(159, 182, 205)
SLATEGRAY4 = Color(108, 123, 139)
SNOW1 = Color(255, 250, 250)
SNOW2 = Color(238, 233, 233)
SNOW3 = Color(205, 201, 201)
SNOW4 = Color(139, 137, 137)
SPRINGGREEN = Color(0, 255, 127)
SPRINGGREEN1 = Color(0, 238, 118)
SPRINGGREEN2 = Color(0, 205, 102)
SPRINGGREEN3 = Color(0, 139, 69)
STEELBLUE = Color(70, 130, 180)
STEELBLUE1 = Color(99, 184, 255)
STEELBLUE2 = Color(92, 172, 238)
STEELBLUE3 = Color(79, 148, 205)
STEELBLUE4 = Color(54, 100, 139)
TAN = Color(210, 180, 140)
TAN1 = Color(255, 165, 79)
TAN2 = Color(238, 154, 73)
TAN3 = Color(205, 133, 63)
TAN4 = Color(139, 90, 43)
TEAL = Color(0, 128, 128)
THISTLE = Color(216, 191, 216)
THISTLE1 = Color(255, 225, 255)
THISTLE2 = Color(238, 210, 238)
THISTLE3 = Color(205, 181, 205)
THISTLE4 = Color(139, 123, 139)
TOMATO1 = Color(255, 99, 71)
TOMATO2 = Color(238, 92, 66)
TOMATO3 = Color(205, 79, 57)
TOMATO4 = Color(139, 54, 38)
TURQUOISE = Color(64, 224, 208)
TURQUOISE1 = Color(0, 245, 255)
TURQUOISE2 = Color(0, 229, 238)
TURQUOISE3 = Color(0, 197, 205)
TURQUOISE4 = Color(0, 134, 139)
TURQUOISEBLUE = Color(0, 199, 140)
VIOLET = Color(238, 130, 238)
VIOLETRED = Color(208, 32, 144)
VIOLETRED1 = Color(255, 62, 150)
VIOLETRED2 = Color(238, 58, 140)
VIOLETRED3 = Color(205, 50, 120)
VIOLETRED4 = Color(139, 34, 82)
WARMGREY = Color(128, 128, 105)
WHEAT = Color(245, 222, 179)
WHEAT1 = Color(255, 231, 186)
WHEAT2 = Color(238, 216, 174)
WHEAT3 = Color(205, 186, 150)
WHEAT4 = Color(139, 126, 102)
WHITE = Color(255, 255, 255)
WHITESMOKE = Color(245, 245, 245)
WHITESMOKE = Color(245, 245, 245)
YELLOW = Color(255, 255, 0)
YELLOW1 = Color(255, 255, 0)
YELLOW2 = Color(238, 238, 0)
YELLOW3 = Color(205, 205, 0)
YELLOW4 = Color(139, 139, 0)

COLORS = {}

# Add colors to colors dictionary
COLORS["aliceblue"] = ALICEBLUE
COLORS["antiquewhite"] = ANTIQUEWHITE
COLORS["antiquewhite1"] = ANTIQUEWHITE1
COLORS["antiquewhite2"] = ANTIQUEWHITE2
COLORS["antiquewhite3"] = ANTIQUEWHITE3
COLORS["antiquewhite4"] = ANTIQUEWHITE4
COLORS["aqua"] = AQUA
COLORS["aquamarine"] = AQUAMARINE1
COLORS["aquamarine1"] = AQUAMARINE1
COLORS["aquamarine2"] = AQUAMARINE2
COLORS["aquamarine3"] = AQUAMARINE3
COLORS["aquamarine4"] = AQUAMARINE4
COLORS["azure"] = AZURE1
COLORS["azure1"] = AZURE1
COLORS["azure2"] = AZURE2
COLORS["azure3"] = AZURE3
COLORS["azure4"] = AZURE4
COLORS["banana"] = BANANA
COLORS["beige"] = BEIGE
COLORS["bisque"] = BISQUE1
COLORS["bisque1"] = BISQUE1
COLORS["bisque2"] = BISQUE2
COLORS["bisque3"] = BISQUE3
COLORS["bisque4"] = BISQUE4
COLORS["black"] = BLACK
COLORS["blanchedalmond"] = BLANCHEDALMOND
COLORS["blue"] = BLUE
COLORS["blue1"] = BLUE
COLORS["blue2"] = BLUE2
COLORS["blue3"] = BLUE3
COLORS["blue4"] = BLUE4
COLORS["blueviolet"] = BLUEVIOLET
COLORS["brick"] = BRICK
COLORS["brown"] = BROWN
COLORS["brown1"] = BROWN1
COLORS["brown2"] = BROWN2
COLORS["brown3"] = BROWN3
COLORS["brown4"] = BROWN4
COLORS["burlywood"] = BURLYWOOD
COLORS["burlywood1"] = BURLYWOOD1
COLORS["burlywood2"] = BURLYWOOD2
COLORS["burlywood3"] = BURLYWOOD3
COLORS["burlywood4"] = BURLYWOOD4
COLORS["burntsienna"] = BURNTSIENNA
COLORS["burntumber"] = BURNTUMBER
COLORS["cadetblue"] = CADETBLUE
COLORS["cadetblue1"] = CADETBLUE1
COLORS["cadetblue2"] = CADETBLUE2
COLORS["cadetblue3"] = CADETBLUE3
COLORS["cadetblue4"] = CADETBLUE4
COLORS["cadmiumorange"] = CADMIUMORANGE
COLORS["cadmiumyellow"] = CADMIUMYELLOW
COLORS["carrot"] = CARROT
COLORS["chartreuse"] = CHARTREUSE1
COLORS["chartreuse1"] = CHARTREUSE1
COLORS["chartreuse2"] = CHARTREUSE2
COLORS["chartreuse3"] = CHARTREUSE3
COLORS["chartreuse4"] = CHARTREUSE4
COLORS["chocolate"] = CHOCOLATE
COLORS["chocolate1"] = CHOCOLATE1
COLORS["chocolate2"] = CHOCOLATE2
COLORS["chocolate3"] = CHOCOLATE3
COLORS["chocolate4"] = CHOCOLATE4
COLORS["cobalt"] = COBALT
COLORS["cobaltgreen"] = COBALTGREEN
COLORS["coldgrey"] = COLDGREY
COLORS["coral"] = CORAL
COLORS["coral1"] = CORAL1
COLORS["coral2"] = CORAL2
COLORS["coral3"] = CORAL3
COLORS["coral4"] = CORAL4
COLORS["cornflowerblue"] = CORNFLOWERBLUE
COLORS["cornsilk"] = CORNSILK1
COLORS["cornsilk1"] = CORNSILK1
COLORS["cornsilk2"] = CORNSILK2
COLORS["cornsilk3"] = CORNSILK3
COLORS["cornsilk4"] = CORNSILK4
COLORS["crimson"] = CRIMSON
COLORS["cyan"] = CYAN1
COLORS["cyan1"] = CYAN1
COLORS["cyan2"] = CYAN2
COLORS["cyan3"] = CYAN3
COLORS["darkgoldenrod"] = DARKGOLDENROD
COLORS["darkgoldenrod1"] = DARKGOLDENROD1
COLORS["darkgoldenrod2"] = DARKGOLDENROD2
COLORS["darkgoldenrod3"] = DARKGOLDENROD3
COLORS["darkgoldenrod4"] = DARKGOLDENROD4
COLORS["darkgray"] = DARKGRAY
COLORS["darkgreen"] = DARKGREEN
COLORS["darkkhaki"] = DARKKHAKI
COLORS["darkolivegreen"] = DARKOLIVEGREEN
COLORS["darkolivegreen1"] = DARKOLIVEGREEN1
COLORS["darkolivegreen2"] = DARKOLIVEGREEN2
COLORS["darkolivegreen3"] = DARKOLIVEGREEN3
COLORS["darkolivegreen4"] = DARKOLIVEGREEN4
COLORS["darkorange"] = DARKORANGE
COLORS["darkorange1"] = DARKORANGE1
COLORS["darkorange2"] = DARKORANGE2
COLORS["darkorange3"] = DARKORANGE3
COLORS["darkorange4"] = DARKORANGE4
COLORS["darkorchid"] = DARKORCHID
COLORS["darkorchid1"] = DARKORCHID1
COLORS["darkorchid2"] = DARKORCHID2
COLORS["darkorchid3"] = DARKORCHID3
COLORS["darkorchid4"] = DARKORCHID4
COLORS["darksalmon"] = DARKSALMON
COLORS["darkseagreen"] = DARKSEAGREEN
COLORS["darkseagreen1"] = DARKSEAGREEN1
COLORS["darkseagreen2"] = DARKSEAGREEN2
COLORS["darkseagreen3"] = DARKSEAGREEN3
COLORS["darkseagreen4"] = DARKSEAGREEN4
COLORS["darkslateblue"] = DARKSLATEBLUE
COLORS["darkslategray"] = DARKSLATEGRAY
COLORS["darkslategray1"] = DARKSLATEGRAY1
COLORS["darkslategray2"] = DARKSLATEGRAY2
COLORS["darkslategray3"] = DARKSLATEGRAY3
COLORS["darkslategray4"] = DARKSLATEGRAY4
COLORS["darkturquoise"] = DARKTURQUOISE
COLORS["darkviolet"] = DARKVIOLET
COLORS["deeppink"] = DEEPPINK1
COLORS["deeppink1"] = DEEPPINK1
COLORS["deeppink2"] = DEEPPINK2
COLORS["deeppink3"] = DEEPPINK3
COLORS["deeppink4"] = DEEPPINK4
COLORS["deepskyblue"] = DEEPSKYBLUE1
COLORS["deepskyblue1"] = DEEPSKYBLUE1
COLORS["deepskyblue2"] = DEEPSKYBLUE2
COLORS["deepskyblue3"] = DEEPSKYBLUE3
COLORS["deepskyblue4"] = DEEPSKYBLUE4
COLORS["dimgray"] = DIMGRAY
COLORS["dodgerblue"] = DODGERBLUE1
COLORS["dodgerblue1"] = DODGERBLUE1
COLORS["dodgerblue2"] = DODGERBLUE2
COLORS["dodgerblue3"] = DODGERBLUE3
COLORS["dodgerblue4"] = DODGERBLUE4
COLORS["eggshell"] = EGGSHELL
COLORS["emeraldgreen"] = EMERALDGREEN
COLORS["firebrick"] = FIREBRICK
COLORS["firebrick1"] = FIREBRICK1
COLORS["firebrick2"] = FIREBRICK2
COLORS["firebrick3"] = FIREBRICK3
COLORS["firebrick4"] = FIREBRICK4
COLORS["flesh"] = FLESH
COLORS["floralwhite"] = FLORALWHITE
COLORS["forestgreen"] = FORESTGREEN
COLORS["gainsboro"] = GAINSBORO
COLORS["ghostwhite"] = GHOSTWHITE
COLORS["gold"] = GOLD1
COLORS["gold1"] = GOLD1
COLORS["gold2"] = GOLD2
COLORS["gold3"] = GOLD3
COLORS["gold4"] = GOLD4
COLORS["goldenrod"] = GOLDENROD
COLORS["goldenrod1"] = GOLDENROD1
COLORS["goldenrod2"] = GOLDENROD2
COLORS["goldenrod3"] = GOLDENROD3
COLORS["goldenrod4"] = GOLDENROD4
COLORS["gray"] = GRAY
COLORS["gray1"] = GRAY1
COLORS["gray10"] = GRAY10
COLORS["gray11"] = GRAY11
COLORS["gray12"] = GRAY12
COLORS["gray13"] = GRAY13
COLORS["gray14"] = GRAY14
COLORS["gray15"] = GRAY15
COLORS["gray16"] = GRAY16
COLORS["gray17"] = GRAY17
COLORS["gray18"] = GRAY18
COLORS["gray19"] = GRAY19
COLORS["gray2"] = GRAY2
COLORS["gray20"] = GRAY20
COLORS["gray21"] = GRAY21
COLORS["gray22"] = GRAY22
COLORS["gray23"] = GRAY23
COLORS["gray24"] = GRAY24
COLORS["gray25"] = GRAY25
COLORS["gray26"] = GRAY26
COLORS["gray27"] = GRAY27
COLORS["gray28"] = GRAY28
COLORS["gray29"] = GRAY29
COLORS["gray3"] = GRAY3
COLORS["gray30"] = GRAY30
COLORS["gray31"] = GRAY31
COLORS["gray32"] = GRAY32
COLORS["gray33"] = GRAY33
COLORS["gray34"] = GRAY34
COLORS["gray35"] = GRAY35
COLORS["gray36"] = GRAY36
COLORS["gray37"] = GRAY37
COLORS["gray38"] = GRAY38
COLORS["gray39"] = GRAY39
COLORS["gray4"] = GRAY4
COLORS["gray40"] = GRAY40
COLORS["gray42"] = GRAY42
COLORS["gray43"] = GRAY43
COLORS["gray44"] = GRAY44
COLORS["gray45"] = GRAY45
COLORS["gray46"] = GRAY46
COLORS["gray47"] = GRAY47
COLORS["gray48"] = GRAY48
COLORS["gray49"] = GRAY49
COLORS["gray5"] = GRAY5
COLORS["gray50"] = GRAY50
COLORS["gray51"] = GRAY51
COLORS["gray52"] = GRAY52
COLORS["gray53"] = GRAY53
COLORS["gray54"] = GRAY54
COLORS["gray55"] = GRAY55
COLORS["gray56"] = GRAY56
COLORS["gray57"] = GRAY57
COLORS["gray58"] = GRAY58
COLORS["gray59"] = GRAY59
COLORS["gray6"] = GRAY6
COLORS["gray60"] = GRAY60
COLORS["gray61"] = GRAY61
COLORS["gray62"] = GRAY62
COLORS["gray63"] = GRAY63
COLORS["gray64"] = GRAY64
COLORS["gray65"] = GRAY65
COLORS["gray66"] = GRAY66
COLORS["gray67"] = GRAY67
COLORS["gray68"] = GRAY68
COLORS["gray69"] = GRAY69
COLORS["gray7"] = GRAY7
COLORS["gray70"] = GRAY70
COLORS["gray71"] = GRAY71
COLORS["gray72"] = GRAY72
COLORS["gray73"] = GRAY73
COLORS["gray74"] = GRAY74
COLORS["gray75"] = GRAY75
COLORS["gray76"] = GRAY76
COLORS["gray77"] = GRAY77
COLORS["gray78"] = GRAY78
COLORS["gray79"] = GRAY79
COLORS["gray8"] = GRAY8
COLORS["gray80"] = GRAY80
COLORS["gray81"] = GRAY81
COLORS["gray82"] = GRAY82
COLORS["gray83"] = GRAY83
COLORS["gray84"] = GRAY84
COLORS["gray85"] = GRAY85
COLORS["gray86"] = GRAY86
COLORS["gray87"] = GRAY87
COLORS["gray88"] = GRAY88
COLORS["gray89"] = GRAY89
COLORS["gray9"] = GRAY9
COLORS["gray90"] = GRAY90
COLORS["gray91"] = GRAY91
COLORS["gray92"] = GRAY92
COLORS["gray93"] = GRAY93
COLORS["gray94"] = GRAY94
COLORS["gray95"] = GRAY95
COLORS["gray97"] = GRAY97
COLORS["gray98"] = GRAY98
COLORS["gray99"] = GRAY99
COLORS["green"] = GREEN
COLORS["green1"] = GREEN1
COLORS["green2"] = GREEN2
COLORS["green3"] = GREEN3
COLORS["green4"] = GREEN4
COLORS["greenyellow"] = GREENYELLOW
COLORS["honeydew"] = HONEYDEW1
COLORS["honeydew1"] = HONEYDEW1
COLORS["honeydew2"] = HONEYDEW2
COLORS["honeydew3"] = HONEYDEW3
COLORS["honeydew4"] = HONEYDEW4
COLORS["hotpink"] = HOTPINK
COLORS["hotpink1"] = HOTPINK1
COLORS["hotpink2"] = HOTPINK2
COLORS["hotpink3"] = HOTPINK3
COLORS["hotpink4"] = HOTPINK4
COLORS["indianred"] = INDIANRED
COLORS["indianred"] = INDIANRED
COLORS["indianred1"] = INDIANRED1
COLORS["indianred2"] = INDIANRED2
COLORS["indianred3"] = INDIANRED3
COLORS["indianred4"] = INDIANRED4
COLORS["indigo"] = INDIGO
COLORS["ivory"] = IVORY1
COLORS["ivory1"] = IVORY1
COLORS["ivory2"] = IVORY2
COLORS["ivory3"] = IVORY3
COLORS["ivory4"] = IVORY4
COLORS["ivoryblack"] = IVORYBLACK
COLORS["khaki"] = KHAKI
COLORS["khaki1"] = KHAKI1
COLORS["khaki2"] = KHAKI2
COLORS["khaki3"] = KHAKI3
COLORS["khaki4"] = KHAKI4
COLORS["lavender"] = LAVENDER
COLORS["lavenderblush"] = LAVENDERBLUSH1
COLORS["lavenderblush1"] = LAVENDERBLUSH1
COLORS["lavenderblush2"] = LAVENDERBLUSH2
COLORS["lavenderblush3"] = LAVENDERBLUSH3
COLORS["lavenderblush4"] = LAVENDERBLUSH4
COLORS["lawngreen"] = LAWNGREEN
COLORS["lemonchiffon"] = LEMONCHIFFON1
COLORS["lemonchiffon1"] = LEMONCHIFFON1
COLORS["lemonchiffon2"] = LEMONCHIFFON2
COLORS["lemonchiffon3"] = LEMONCHIFFON3
COLORS["lemonchiffon4"] = LEMONCHIFFON4
COLORS["lightblue"] = LIGHTBLUE
COLORS["lightblue1"] = LIGHTBLUE1
COLORS["lightblue2"] = LIGHTBLUE2
COLORS["lightblue3"] = LIGHTBLUE3
COLORS["lightblue4"] = LIGHTBLUE4
COLORS["lightcoral"] = LIGHTCORAL
COLORS["lightcyan"] = LIGHTCYAN1
COLORS["lightcyan1"] = LIGHTCYAN1
COLORS["lightcyan2"] = LIGHTCYAN2
COLORS["lightcyan3"] = LIGHTCYAN3
COLORS["lightcyan4"] = LIGHTCYAN4
COLORS["lightgoldenrod"] = LIGHTGOLDENROD1
COLORS["lightgoldenrod1"] = LIGHTGOLDENROD1
COLORS["lightgoldenrod2"] = LIGHTGOLDENROD2
COLORS["lightgoldenrod3"] = LIGHTGOLDENROD3
COLORS["lightgoldenrod4"] = LIGHTGOLDENROD4
COLORS["lightgoldenrodyellow"] = LIGHTGOLDENRODYELLOW
COLORS["lightgrey"] = LIGHTGREY
COLORS["lightpink"] = LIGHTPINK
COLORS["lightpink1"] = LIGHTPINK1
COLORS["lightpink2"] = LIGHTPINK2
COLORS["lightpink3"] = LIGHTPINK3
COLORS["lightpink4"] = LIGHTPINK4
COLORS["lightsalmon"] = LIGHTSALMON1
COLORS["lightsalmon1"] = LIGHTSALMON1
COLORS["lightsalmon2"] = LIGHTSALMON2
COLORS["lightsalmon3"] = LIGHTSALMON3
COLORS["lightsalmon4"] = LIGHTSALMON4
COLORS["lightseagreen"] = LIGHTSEAGREEN
COLORS["lightskyblue"] = LIGHTSKYBLUE
COLORS["lightskyblue1"] = LIGHTSKYBLUE1
COLORS["lightskyblue2"] = LIGHTSKYBLUE2
COLORS["lightskyblue3"] = LIGHTSKYBLUE3
COLORS["lightskyblue4"] = LIGHTSKYBLUE4
COLORS["lightslateblue"] = LIGHTSLATEBLUE
COLORS["lightslategray"] = LIGHTSLATEGRAY
COLORS["lightsteelblue"] = LIGHTSTEELBLUE
COLORS["lightsteelblue1"] = LIGHTSTEELBLUE1
COLORS["lightsteelblue2"] = LIGHTSTEELBLUE2
COLORS["lightsteelblue3"] = LIGHTSTEELBLUE3
COLORS["lightsteelblue4"] = LIGHTSTEELBLUE4
COLORS["lightyellow"] = LIGHTYELLOW1
COLORS["lightyellow1"] = LIGHTYELLOW1
COLORS["lightyellow2"] = LIGHTYELLOW2
COLORS["lightyellow3"] = LIGHTYELLOW3
COLORS["lightyellow4"] = LIGHTYELLOW4
COLORS["limegreen"] = LIMEGREEN
COLORS["linen"] = LINEN
COLORS["magenta"] = MAGENTA
COLORS["magenta1"] = MAGENTA2
COLORS["magenta2"] = MAGENTA2
COLORS["magenta3"] = MAGENTA3
COLORS["magenta4"] = MAGENTA4
COLORS["manganeseblue"] = MANGANESEBLUE
COLORS["maroon"] = MAROON
COLORS["maroon1"] = MAROON1
COLORS["maroon2"] = MAROON2
COLORS["maroon3"] = MAROON3
COLORS["maroon4"] = MAROON4
COLORS["mediumorchid"] = MEDIUMORCHID
COLORS["mediumorchid1"] = MEDIUMORCHID1
COLORS["mediumorchid2"] = MEDIUMORCHID2
COLORS["mediumorchid3"] = MEDIUMORCHID3
COLORS["mediumorchid4"] = MEDIUMORCHID4
COLORS["mediumpurple"] = MEDIUMPURPLE
COLORS["mediumpurple1"] = MEDIUMPURPLE1
COLORS["mediumpurple2"] = MEDIUMPURPLE2
COLORS["mediumpurple3"] = MEDIUMPURPLE3
COLORS["mediumpurple4"] = MEDIUMPURPLE4
COLORS["mediumseagreen"] = MEDIUMSEAGREEN
COLORS["mediumslateblue"] = MEDIUMSLATEBLUE
COLORS["mediumspringgreen"] = MEDIUMSPRINGGREEN
COLORS["mediumturquoise"] = MEDIUMTURQUOISE
COLORS["mediumvioletred"] = MEDIUMVIOLETRED
COLORS["melon"] = MELON
COLORS["midnightblue"] = MIDNIGHTBLUE
COLORS["mint"] = MINT
COLORS["mintcream"] = MINTCREAM
COLORS["mistyrose"] = MISTYROSE1
COLORS["mistyrose1"] = MISTYROSE1
COLORS["mistyrose2"] = MISTYROSE2
COLORS["mistyrose3"] = MISTYROSE3
COLORS["mistyrose4"] = MISTYROSE4
COLORS["moccasin"] = MOCCASIN
COLORS["navajowhite"] = NAVAJOWHITE1
COLORS["navajowhite1"] = NAVAJOWHITE1
COLORS["navajowhite2"] = NAVAJOWHITE2
COLORS["navajowhite3"] = NAVAJOWHITE3
COLORS["navajowhite4"] = NAVAJOWHITE4
COLORS["navy"] = NAVY
COLORS["oldlace"] = OLDLACE
COLORS["olive"] = OLIVE
COLORS["olivedrab"] = OLIVEDRAB
COLORS["olivedrab1"] = OLIVEDRAB1
COLORS["olivedrab2"] = OLIVEDRAB2
COLORS["olivedrab3"] = OLIVEDRAB3
COLORS["olivedrab4"] = OLIVEDRAB4
COLORS["orange"] = ORANGE
COLORS["orange1"] = ORANGE1
COLORS["orange2"] = ORANGE2
COLORS["orange3"] = ORANGE3
COLORS["orange4"] = ORANGE4
COLORS["orangered"] = ORANGERED1
COLORS["orangered1"] = ORANGERED1
COLORS["orangered2"] = ORANGERED2
COLORS["orangered3"] = ORANGERED3
COLORS["orangered4"] = ORANGERED4
COLORS["orchid"] = ORCHID
COLORS["orchid1"] = ORCHID1
COLORS["orchid2"] = ORCHID2
COLORS["orchid3"] = ORCHID3
COLORS["orchid4"] = ORCHID4
COLORS["palegoldenrod"] = PALEGOLDENROD
COLORS["palegreen"] = PALEGREEN
COLORS["palegreen1"] = PALEGREEN1
COLORS["palegreen2"] = PALEGREEN2
COLORS["palegreen3"] = PALEGREEN3
COLORS["palegreen4"] = PALEGREEN4
COLORS["paleturquoise1"] = PALETURQUOISE1
COLORS["paleturquoise2"] = PALETURQUOISE2
COLORS["paleturquoise3"] = PALETURQUOISE3
COLORS["paleturquoise4"] = PALETURQUOISE4
COLORS["palevioletred"] = PALEVIOLETRED
COLORS["palevioletred1"] = PALEVIOLETRED1
COLORS["palevioletred2"] = PALEVIOLETRED2
COLORS["palevioletred3"] = PALEVIOLETRED3
COLORS["palevioletred4"] = PALEVIOLETRED4
COLORS["papayawhip"] = PAPAYAWHIP
COLORS["peachpuff1"] = PEACHPUFF1
COLORS["peachpuff2"] = PEACHPUFF2
COLORS["peachpuff3"] = PEACHPUFF3
COLORS["peachpuff4"] = PEACHPUFF4
COLORS["peacock"] = PEACOCK
COLORS["pink"] = PINK
COLORS["pink1"] = PINK1
COLORS["pink2"] = PINK2
COLORS["pink3"] = PINK3
COLORS["pink4"] = PINK4
COLORS["plum"] = PLUM
COLORS["plum1"] = PLUM1
COLORS["plum2"] = PLUM2
COLORS["plum3"] = PLUM3
COLORS["plum4"] = PLUM4
COLORS["powderblue"] = POWDERBLUE
COLORS["purple"] = PURPLE
COLORS["purple1"] = PURPLE1
COLORS["purple2"] = PURPLE2
COLORS["purple3"] = PURPLE3
COLORS["purple4"] = PURPLE4
COLORS["raspberry"] = RASPBERRY
COLORS["rawsienna"] = RAWSIENNA
COLORS["red"] = RED
COLORS["red1"] = RED1
COLORS["red2"] = RED2
COLORS["red3"] = RED3
COLORS["red4"] = RED4
COLORS["rosybrown"] = ROSYBROWN
COLORS["rosybrown1"] = ROSYBROWN1
COLORS["rosybrown2"] = ROSYBROWN2
COLORS["rosybrown3"] = ROSYBROWN3
COLORS["rosybrown4"] = ROSYBROWN4
COLORS["royalblue"] = ROYALBLUE
COLORS["royalblue1"] = ROYALBLUE1
COLORS["royalblue2"] = ROYALBLUE2
COLORS["royalblue3"] = ROYALBLUE3
COLORS["royalblue4"] = ROYALBLUE4
COLORS["salmon"] = SALMON
COLORS["salmon1"] = SALMON1
COLORS["salmon2"] = SALMON2
COLORS["salmon3"] = SALMON3
COLORS["salmon4"] = SALMON4
COLORS["sandybrown"] = SANDYBROWN
COLORS["sapgreen"] = SAPGREEN
COLORS["seagreen1"] = SEAGREEN1
COLORS["seagreen2"] = SEAGREEN2
COLORS["seagreen3"] = SEAGREEN3
COLORS["seagreen4"] = SEAGREEN4
COLORS["seashell1"] = SEASHELL1
COLORS["seashell2"] = SEASHELL2
COLORS["seashell3"] = SEASHELL3
COLORS["seashell4"] = SEASHELL4
COLORS["sepia"] = SEPIA
COLORS["sgibeet"] = SGIBEET
COLORS["sgibrightgray"] = SGIBRIGHTGRAY
COLORS["sgichartreuse"] = SGICHARTREUSE
COLORS["sgidarkgray"] = SGIDARKGRAY
COLORS["sgigray12"] = SGIGRAY12
COLORS["sgigray16"] = SGIGRAY16
COLORS["sgigray32"] = SGIGRAY32
COLORS["sgigray36"] = SGIGRAY36
COLORS["sgigray52"] = SGIGRAY52
COLORS["sgigray56"] = SGIGRAY56
COLORS["sgigray72"] = SGIGRAY72
COLORS["sgigray76"] = SGIGRAY76
COLORS["sgigray92"] = SGIGRAY92
COLORS["sgigray96"] = SGIGRAY96
COLORS["sgilightblue"] = SGILIGHTBLUE
COLORS["sgilightgray"] = SGILIGHTGRAY
COLORS["sgiolivedrab"] = SGIOLIVEDRAB
COLORS["sgisalmon"] = SGISALMON
COLORS["sgislateblue"] = SGISLATEBLUE
COLORS["sgiteal"] = SGITEAL
COLORS["sienna"] = SIENNA
COLORS["sienna1"] = SIENNA1
COLORS["sienna2"] = SIENNA2
COLORS["sienna3"] = SIENNA3
COLORS["sienna4"] = SIENNA4
COLORS["silver"] = SILVER
COLORS["skyblue"] = SKYBLUE
COLORS["skyblue1"] = SKYBLUE1
COLORS["skyblue2"] = SKYBLUE2
COLORS["skyblue3"] = SKYBLUE3
COLORS["skyblue4"] = SKYBLUE4
COLORS["slateblue"] = SLATEBLUE
COLORS["slateblue1"] = SLATEBLUE1
COLORS["slateblue2"] = SLATEBLUE2
COLORS["slateblue3"] = SLATEBLUE3
COLORS["slateblue4"] = SLATEBLUE4
COLORS["slategray"] = SLATEGRAY
COLORS["slategray1"] = SLATEGRAY1
COLORS["slategray2"] = SLATEGRAY2
COLORS["slategray3"] = SLATEGRAY3
COLORS["slategray4"] = SLATEGRAY4
COLORS["snow"] = SNOW1
COLORS["snow1"] = SNOW1
COLORS["snow2"] = SNOW2
COLORS["snow3"] = SNOW3
COLORS["snow4"] = SNOW4
COLORS["springgreen"] = SPRINGGREEN
COLORS["springgreen1"] = SPRINGGREEN1
COLORS["springgreen2"] = SPRINGGREEN2
COLORS["springgreen3"] = SPRINGGREEN3
COLORS["steelblue"] = STEELBLUE
COLORS["steelblue1"] = STEELBLUE1
COLORS["steelblue2"] = STEELBLUE2
COLORS["steelblue3"] = STEELBLUE3
COLORS["steelblue4"] = STEELBLUE4
COLORS["tan"] = TAN
COLORS["tan1"] = TAN1
COLORS["tan2"] = TAN2
COLORS["tan3"] = TAN3
COLORS["tan4"] = TAN4
COLORS["teal"] = TEAL
COLORS["thistle"] = THISTLE
COLORS["thistle1"] = THISTLE1
COLORS["thistle2"] = THISTLE2
COLORS["thistle3"] = THISTLE3
COLORS["thistle4"] = THISTLE4
COLORS["tomato"] = TOMATO1
COLORS["tomato1"] = TOMATO1
COLORS["tomato2"] = TOMATO2
COLORS["tomato3"] = TOMATO3
COLORS["tomato4"] = TOMATO4
COLORS["turquoise"] = TURQUOISE
COLORS["turquoise1"] = TURQUOISE1
COLORS["turquoise2"] = TURQUOISE2
COLORS["turquoise3"] = TURQUOISE3
COLORS["turquoise4"] = TURQUOISE4
COLORS["turquoiseblue"] = TURQUOISEBLUE
COLORS["violet"] = VIOLET
COLORS["violetred"] = VIOLETRED
COLORS["violetred1"] = VIOLETRED1
COLORS["violetred2"] = VIOLETRED2
COLORS["violetred3"] = VIOLETRED3
COLORS["violetred4"] = VIOLETRED4
COLORS["warmgrey"] = WARMGREY
COLORS["wheat"] = WHEAT
COLORS["wheat1"] = WHEAT1
COLORS["wheat2"] = WHEAT2
COLORS["wheat3"] = WHEAT3
COLORS["wheat4"] = WHEAT4
COLORS["white"] = WHITE
COLORS["whitesmoke"] = WHITESMOKE
COLORS["whitesmoke"] = WHITESMOKE
COLORS["yellow"] = YELLOW
COLORS["yellow1"] = YELLOW1
COLORS["yellow2"] = YELLOW2
COLORS["yellow3"] = YELLOW3
COLORS["yellow4"] = YELLOW4

def parse_color(value):
    if value is not None:

        # is it a Color object?
        if isinstance(value, Color):
            return value

        # is the color a string
        elif isinstance(value, str):
            # strip the color of white space
            value = value.strip()

            # if it starts with a # check it is a valid color
            if value[0] == "#":

                # check its format
                if len(value) != 7 and len(value) != 9:
                    raise ValueError("{} is not a valid # color, it must be in the format #rrggbb or #rrggbbaa".format(value))
                else:
                    # add the alpha if required
                    if len(value) == 7:
                        value = value + "ff"

                    hex_values = (value[1:3], value[3:5], value[5:7], value[7:9])
                
                    # check hex values are between 00 and ff
                    int_values = []
                    for hex_color in hex_values:
                        try:
                            int_color = int(hex_color, 16)
                            int_values.append(int_color)
                        except: 
                            raise ValueError("{} is not a valid value, it must be hex 00 - ff".format(hex_color))

                        if not (0 <= int_color <= 255):
                            raise ValueError("{} is not a valid color value, it must be 00 - ff".format(hex_color))

                    return Color(red=int_values[0], green=int_values[1], blue=int_values[2], alpha=int_values[3])
            
            else:
                # does the color exist in the dictionary of colors
                if value.lower() in COLORS:
                    return COLORS[value.lower()]
                else:
                    raise ValueError("'{}' is not a valid color value.")

        # if the color is not a string or a Color object, maybe its a list or tuple - try and convert it
        else:
            # get the number of colors and check it is iterable
            try:
                no_of_colors = len(value)
            except:
                raise ValueError("A color must be a Color object, string or list of values (red, green, blue) or (alpha, red, green, blue)") 

            if not (3 <= no_of_colors <= 4):
                raise ValueError("A color must contain 3 or 4 values (red, green, blue) or (red, green, blue, alpha)")
            
            # check the color values are between 0 and 255
            for c in value:
                if not (0 <= c <= 255):
                    raise ValueError("{} is not a valid color value, it must be 0 - 255")
                    
            if no_of_colors == 3:
                return Color(red=value[0], green=value[1], blue=value[2])
            else:
                return Color(red=value[0], green=value[1], blue=value[2], alpha=value[3])




from .btcomm import BluetoothServer, BluetoothClient, BluetoothAdapter
from .dot import BlueDot
from .threads import WrapThread
from .constants import PROTOCOL_VERSION

CLIENT_NAME = "Mock client"

class MockBluetoothAdapter(BluetoothAdapter):
    def __init__(self, device = "mock0", address = "00:00:00:00:00:00"):
        self._device = device
        self._address = address
        self._powered = True
        self._discoverable = False
        self._pairable = False
        self._pairing_thread = None

    @property
    def powered(self):
        return self._powered

    @powered.setter
    def powered(self, value):
        self._powered = value

    @property
    def discoverable(self):
        return self._discoverable

    @discoverable.setter
    def discoverable(self, value):
        self._discoverable = value

    @property
    def pairable(self):
        return self._pairable

    @pairable.setter
    def pairable(self, value):
        self._pairable = value

    @property
    def paired_devices(self):
        return [["01:01:01:01:01:01", "mock_device_1"], ["02:02:02:02:02:02", "mock_device_2"]]


class MockBluetoothServer(BluetoothServer):
    """
    :class:`MockBluetoothServer` inherits from
    :class:`~.btcomm.BluetoothServer` but overrides ``__init__``, :meth:`start`
    , :meth:`stop` and :meth:`send_raw` to create a :class:`MockBluetoothServer` which can
    be used for testing and debugging.
    """
    def __init__(self,
        data_received_callback,
        auto_start = True,
        device = "mock0",
        port = 1,
        encoding = "utf-8",
        power_up_device = False,
        when_client_connects = None,
        when_client_disconnects = None):

        super(MockBluetoothServer, self).__init__(
            data_received_callback,
            auto_start,
            device,
            port,
            encoding,
            power_up_device,
            when_client_connects,
            when_client_disconnects)

        self._mock_client = None

    def start(self):
        self._running = True

    def stop(self):
        self._running = False

    def mock_client_connected(self, mock_client = None):
        """
        Simulates a client connected to the :class:`~.btcomm.BluetoothServer`.
        
        :param MockBluetoothClient mock_client:
            The mock client to interact with, defaults to `None`. If `None`, 
            client address is set to '99:99:99:99:99:99'
        """
        self._mock_client = mock_client

        if not self._client_connected:
            if self._mock_client is None:
                client_address = "99:99:99:99:99:99"
            else:
                client_address = self._mock_client.adapter.address
            self._client_connected = True
            self._client_info = (client_address, self.port)
            #call the call back
            if self.when_client_connects:
                WrapThread(target=self.when_client_connects).start()

    def mock_client_disconnected(self):
        """
        Simulates a client disconnecting from the
        :class:`~.btcomm.BluetoothServer`.
        """
        if self._client_connected:
            self._client_connected = False
            self._client_info = None
            if self._when_client_disconnects:
                WrapThread(target=self.when_client_disconnects).start()

    def mock_client_sending_data(self, data):
        """
        Simulates a client sending data to the
        :class:`~.btcomm.BluetoothServer`.
        """
        if self._client_connected:
            self._data_received_callback(data)

    def _send_data(self, data):
        if self._mock_client is not None:
            # call the data received callback
            if self._encoding:
                data = data.decode(self._encoding)
            self._mock_client.mock_server_sending_data(data)

    def _setup_adapter(self, device):
        self._adapter = MockBluetoothAdapter(device)


class MockBluetoothClient(BluetoothClient):
    """
    :class:`MockBluetoothClient` inherits from
    :class:`~.btcomm.BluetoothClient` but overrides ``__init__``, :meth:`connect`
    and :meth:`send_raw` to create a :class:`MockBluetoothServer` which can
    be used for testing and debugging.

    Note - the `server` parameter should be an instance of :class:`MockBluetoothServer`.
    """
    def __init__(self,
        server,
        data_received_callback,
        port = 1,
        device = "mock1",
        encoding = "utf-8",
        power_up_device = False,
        auto_connect = True):

        super(MockBluetoothClient, self).__init__(
            server,
            data_received_callback,
            port,
            device,
            encoding,
            power_up_device,
            auto_connect)

    def connect(self):
        """
        Connect to a Bluetooth server.
        """
        self._server.mock_client_connected(self)
        self._connected = True

    def disconnect(self):
        """
        Disconnect from a Bluetooth server.
        """
        self._server.mock_client_disconnected()
        self._connected = False

    def mock_server_sending_data(self, data):
        """
        Simulates a server sending data to the
        :class:`~.btcomm.BluetoothClient`.
        """
        if self._connected:
            self._data_received_callback(data)

    def _send_data(self, data):
        # send data to the server
        # call the data received callback
        if self._encoding:
            data = data.decode(self._encoding)
        self._server.mock_client_sending_data(data)

    def _setup_adapter(self, device):
        self._adapter = MockBluetoothAdapter(device, address = "11:11:11:11:11:11")

class MockBlueDot(BlueDot):
    """
    :class:`MockBlueDot` inherits from :class:`BlueDot` but overrides
    :meth:`_create_server`, to create a :class:`~.mock.MockBluetoothServer`
    which can be used for testing and debugging.
    """
    def _create_server(self):
        self._server = MockBluetoothServer(
                self._data_received,
                when_client_connects = self._client_connected,
                when_client_disconnects = self._client_disconnected,
                device = self.device,
                port = self.port,
                power_up_device = self._power_up_device,
                auto_start = False)

    def mock_client_connected(self):
        """
        Simulates a client connecting to the Blue Dot.

        :param string client_address:
            The mock client mac address, defaults to '11:11:11:11:11:11'
        """
        self._server.mock_client_connected()
        # send protocol version to server
        self._server.mock_client_sending_data("3,{},{}\n".format(PROTOCOL_VERSION, CLIENT_NAME))

    def mock_client_disconnected(self):
        """
        Simulates a client disconnecting from the Blue Dot.
        """
        self._server.mock_client_disconnected()

    def mock_blue_dot_pressed(self, x, y):
        """
        Simulates the Blue Dot being pressed.

        :param int x:
            The x position where the mock Blue Dot was pressed

        :param int y:
            The y position where the mock Blue Dot was pressed
        """
        self._server.mock_client_sending_data("1,{},{}\n".format(x, y))

    def mock_blue_dot_released(self, x, y):
        """
        Simulates the Blue Dot being released.

        :param int x:
            The x position where the mock Blue Dot was released

        :param int y:
            The y position where the mock Blue Dot was released
        """
        self._server.mock_client_sending_data("0,{},{}\n".format(x, y))

    def mock_blue_dot_moved(self, x, y):
        """
        Simulates the Blue Dot being moved.

        :param int x:
            The x position where the mock Blue Dot was moved too

        :param int y:
            The y position where the mock Blue Dot was moved too
        """
        self._server.mock_client_sending_data("2,{},{}\n".format(x, y))

    def launch_mock_app(self):
        """
        Launches a mock Blue Dot app.

        The mock app reacts to mouse clicks and movement and calls the mock blue
        dot methods to simulates presses.

        This is useful for testing, allowing you to interact with Blue Dot without
        having to script mock functions.

        The mock app uses pygame which will need to be installed.
        """
        self._mock_app_thread = WrapThread(target=self._launch_mock_app)
        self._mock_app_thread.start()

    def _launch_mock_app(self):
        #imported here, so pygame is only a pre-requisite for the mock app
        from .app import BlueDotClient, ButtonScreen

        class MockBlueDotClient(BlueDotClient):
            def _run(self):
                button_screen = MockButtonScreen(self._screen, self._font, self._device, self._server, self._width, self._height)
                button_screen.run()

        class MockButtonScreen(ButtonScreen):
            def _connect(self):
                self.bt_client = MockBluetoothClient(self.server, self._data_received, device = self.device, auto_connect = True)

        MockBlueDotClient("mock2", self._server, None, None, None)



from __future__ import unicode_literals

import socket
import sys
import errno

from .utils import (
    register_spp,
    get_mac,
    get_adapter_powered_status,
    get_adapter_discoverable_status,
    get_adapter_pairable_status,
    get_paired_devices,
    device_pairable,
    device_discoverable,
    device_powered,
)

from .threads import WrapThread

BLUETOOTH_TIMEOUT = 0.01


class BluetoothAdapter(object):
    """
    Represents and allows interaction with a Bluetooth Adapter.

    The following example will get the Bluetooth adapter, print its powered status
    and any paired devices::

        a = BluetoothAdapter()
        print("Powered = {}".format(a.powered))
        print(a.paired_devices)

    :param str device:
        The Bluetooth device to be used, the default is "hci0", if your device
        only has 1 Bluetooth adapter this shouldn't need to be changed.
    """
    def __init__(self, device = "hci0"):
        self._device = device
        self._address = get_mac(self._device)
        self._pairing_thread = None

    @property
    def device(self):
        """
        The Bluetooth device name. This defaults to "hci0".
        """
        return self._device

    @property
    def address(self):
        """
        The `MAC address`_ of the Bluetooth adapter.

        .. _MAC address: https://en.wikipedia.org/wiki/MAC_address
        """
        return self._address

    @property
    def powered(self):
        """
        Set to ``True`` to power on the Bluetooth adapter.

        Depending on how Bluetooth has been powered down, you may need to use
        :command:`rfkill` to unblock Bluetooth to give permission to bluez to power on Bluetooth::

            sudo rfkill unblock bluetooth
        """
        return get_adapter_powered_status(self._device)

    @powered.setter
    def powered(self, value):
        device_powered(self._device, value)

    @property
    def discoverable(self):
        """
        Set to ``True`` to make the Bluetooth adapter discoverable.
        """
        return get_adapter_discoverable_status(self._device)

    @discoverable.setter
    def discoverable(self, value):
        device_discoverable(self._device, value)

    @property
    def pairable(self):
        """
        Set to ``True`` to make the Bluetooth adapter pairable.
        """
        return get_adapter_pairable_status(self._device)

    @pairable.setter
    def pairable(self, value):
        device_pairable(self._device, value)

    @property
    def paired_devices(self):
        """
        Returns a sequence of devices paired with this adapater
        :code:`[(mac_address, name), (mac_address, name), ...]`::

            a = BluetoothAdapter()
            devices = a.paired_devices
            for d in devices:
                device_address = d[0]
                device_name = d[1]
        """
        return get_paired_devices(self._device)

    def allow_pairing(self, timeout = 60):
        """
        Put the adapter into discoverable and pairable mode.

        :param int timeout:
            The time in seconds the adapter will remain pairable. If set to ``None``
            the device will be discoverable and pairable indefinetly.
        """
        #if a pairing thread is already running, stop it and restart
        if self._pairing_thread:
            if self._pairing_thread.is_alive:
                self._pairing_thread.stop()

        #make the adapter pairable
        self.pairable = True
        self.discoverable = True

        if timeout != None:
            #start the pairing thread
            self._pairing_thread = WrapThread(target=self._expire_pairing, args=(timeout, ))
            self._pairing_thread.start()

    def _expire_pairing(self, timeout):
        #wait till the timeout or the thread is stopped
        self._pairing_thread.stopping.wait(timeout)
        self.discoverable = False
        self.pairable = False


class BluetoothServer(object):
    """
    Creates a Bluetooth server which will allow connections and accept incoming
    RFCOMM serial data.

    When data is received by the server it is passed to a callback function
    which must be specified at initiation.

    The following example will create a Bluetooth server which will wait for a
    connection and print any data it receives and send it back to the client::

        from bluedot.btcomm import BluetoothServer
        from signal import pause

        def data_received(data):
            print(data)
            s.send(data)

        s = BluetoothServer(data_received)
        pause()

    :param data_received_callback:
        A function reference should be passed, this function will be called when
        data is received by the server.  The function should accept a single parameter
        which when called will hold the data received. Set to ``None`` if  received
        data is not required.

    :param bool auto_start:
        If ``True`` (the default), the Bluetooth server will be automatically started
        on initialisation, if ``False``, the method ``start`` will need to be called
        before connections will be accepted.

    :param str device:
        The Bluetooth device the server should use, the default is "hci0", if
        your device only has 1 Bluetooth adapter this shouldn't need to be changed.

    :param int port:
        The Bluetooth port the server should use, the default is 1.

    :param str encoding:
        The encoding standard to be used when sending and receiving byte data. The default is
        "utf-8".  If set to ``None`` no encoding is done and byte data types should be used.

    :param bool power_up_device:
        If ``True``, the Bluetooth device will be powered up (if required) when the
        server starts. The default is ``False``.

        Depending on how Bluetooth has been powered down, you may need to use :command:`rfkill`
        to unblock Bluetooth to give permission to bluez to power on Bluetooth::

            sudo rfkill unblock bluetooth

    :param when_client_connects:
        A function reference which will be called when a client connects. If ``None``
        (the default), no notification will be given when a client connects

    :param when_client_disconnects:
        A function reference which will be called when a client disconnects. If ``None``
        (the default), no notification will be given when a client disconnects

    """
    def __init__(self,
        data_received_callback,
        auto_start = True,
        device = "hci0",
        port = 1,
        encoding = "utf-8",
        power_up_device = False,
        when_client_connects = None,
        when_client_disconnects = None):

        self._setup_adapter(device)

        self._data_received_callback = data_received_callback
        self._port = port
        self._encoding = encoding
        self._power_up_device = power_up_device
        self._when_client_connects = when_client_connects
        self._when_client_disconnects = when_client_disconnects

        self._running = False
        self._client_connected = False
        self._server_sock = None
        self._client_info = None
        self._client_sock = None

        self._conn_thread = None

        if auto_start:
            self.start()

    @property
    def device(self):
        """
        The Bluetooth device the server is using. This defaults to "hci0".
        """
        return self.adapter.device

    @property
    def adapter(self):
        """
        A :class:`BluetoothAdapter` object which represents the Bluetooth device
        the server is using.
        """
        return self._adapter

    @property
    def port(self):
        """
        The port the server is using. This defaults to 1.
        """
        return self._port

    @property
    def encoding(self):
        """
        The encoding standard the server is using. This defaults to "utf-8".
        """
        return self._encoding

    @property
    def running(self):
        """
        Returns a ``True`` if the server is running.
        """
        return self._running

    @property
    def server_address(self):
        """
        The `MAC address`_ of the device the server is using.

        .. _MAC address: https://en.wikipedia.org/wiki/MAC_address
        """
        return self.adapter.address

    @property
    def client_address(self):
        """
        The `MAC address`_ of the client connected to the server. Returns
        ``None`` if no client is connected.

        .. _MAC address: https://en.wikipedia.org/wiki/MAC_address
        """
        if self._client_info:
            return self._client_info[0]
        else:
            return None

    @property
    def client_connected(self):
        """
        Returns ``True`` if a client is connected.
        """
        return self._client_connected

    @property
    def data_received_callback(self):
        """
        Sets or returns the function which is called when data is received by the server.

        The function should accept a single parameter which when called will hold
        the data received. Set to ``None`` if received data is not required.
        """
        return self._data_received_callback

    @data_received_callback.setter
    def data_received_callback(self, value):
        self._data_received_callback = value

    @property
    def when_client_connects(self):
        """
        Sets or returns the function which is called when a client connects.
        """
        return self._when_client_connects

    @when_client_connects.setter
    def when_client_connects(self, value):
        self._when_client_connects = value

    @property
    def when_client_disconnects(self):
        """
        Sets or returns the function which is called when a client disconnects.
        """
        return self._when_client_disconnects

    @when_client_disconnects.setter
    def when_client_disconnects(self, value):
        self._when_client_disconnects = value

    def start(self):
        """
        Starts the Bluetooth server if its not already running. The server needs to be started before
        connections can be made.
        """
        if not self._running:

            if self._power_up_device:
                self.adapter.powered = True

            if not self.adapter.powered:
                raise Exception("Bluetooth device {} is turned off".format(self.adapter.device))

            #register the serial port profile with Bluetooth
            register_spp(self._port)

            #start Bluetooth server
            #open the Bluetooth socket
            self._server_sock = socket.socket(socket.AF_BLUETOOTH, socket.SOCK_STREAM, socket.BTPROTO_RFCOMM)
            self._server_sock.settimeout(BLUETOOTH_TIMEOUT)
            try:
                self._server_sock.bind((self.server_address, self.port))
            except (socket.error, OSError) as e:
                if e.errno == errno.EADDRINUSE:
                    print("Bluetooth address {} is already in use - is the server already running?".format(self.server_address))
                raise e
            self._server_sock.listen(1)

            #wait for client connection
            self._conn_thread = WrapThread(target=self._wait_for_connection)
            self._conn_thread.start()

            self._running = True

    def stop(self):
        """
        Stops the Bluetooth server if its running.
        """
        if self._running:
            if self._conn_thread:
                self._conn_thread.stop()
                self._conn_thread = None

    def send(self, data):
        """
        Send data to a connected Bluetooth client

        :param str data:
            The data to be sent.
        """
        if self._client_connected:
            if self._encoding is not None:
                data = data.encode(self._encoding)
            try:
                self._send_data(data)
            except IOError as e:
                self._handle_bt_error(e)

    def _send_data(self, data):
        """
        Send raw data to the client.
        
        :param bytes data:
            The data to be sent.
        """
        self._client_sock.send(data)

    def disconnect_client(self):
        """
        Disconnects the client if connected. Returns `True` if a client was disconnected.
        """
        if self._client_connected:    
            self._client_connected = False
            
            # call the callback
            if self.when_client_disconnects:
                WrapThread(target=self.when_client_disconnects).start()
            
            return True
        
        else:
            return False

    def _setup_adapter(self, device):
        self._adapter = BluetoothAdapter(device)
    
    def _wait_for_connection(self):
        #keep going until the server is stopped
        while not self._conn_thread.stopping.is_set():
            #wait for connection
            self._client_connected = False
            while not self._conn_thread.stopping.is_set():
                try:
                    # accept() will timeout after BLUETOOTH_TIMEOUT seconds
                    self._client_sock, self._client_info = self._server_sock.accept()
                    self._client_connected = True
                    break
                except socket.timeout as e:
                    self._handle_bt_error(e)

            #did a client connect?
            if self._client_connected:
                #call the call back
                if self.when_client_connects:
                    WrapThread(target=self.when_client_connects).start()

                #read data
                self._read()

        #server has been stopped
        self._server_sock.close()
        self._server_sock = None
        self._running = False

    def _read(self):
        #read until the server is stopped or the client disconnects
        while self._client_connected:
            #read data from Bluetooth socket
            try:
                data = self._client_sock.recv(1024, socket.MSG_DONTWAIT)
            except IOError as e:
                self._handle_bt_error(e)
                data = b""
            if data:
                if self._data_received_callback:
                    if self._encoding:
                        data = data.decode(self._encoding)
                    self.data_received_callback(data)
            if self._conn_thread.stopping.wait(BLUETOOTH_TIMEOUT):
                break

        #close the client socket
        self._client_sock.close()
        self._client_sock = None
        self._client_info = None
        self._client_connected = False

    def _handle_bt_error(self, bt_error):
        assert isinstance(bt_error, IOError)
        #'timed out' is caused by the wait_for_connection loop
        if isinstance(bt_error, socket.timeout):
            pass
        #'resource unavailable' is when data cannot be read because there is nothing in the buffer
        elif bt_error.errno == errno.EAGAIN:
            pass
        #'connection reset' is caused when the client disconnects
        elif bt_error.errno == errno.ECONNRESET:
            self.disconnect_client()
        #'conection timeout' is caused when the server can no longer connect to read from the client
        # (perhaps the client has gone out of range)
        elif bt_error.errno == errno.ETIMEDOUT:
            self.disconnect_client()
        else:
            raise bt_error


class BluetoothClient():
    """
    Creates a Bluetooth client which can send data to a server using RFCOMM Serial Data.

    The following example will create a Bluetooth client which will connect to a paired
    device called "raspberrypi", send "helloworld" and print any data is receives::

        from bluedot.btcomm import BluetoothClient
        from signal import pause

        def data_received(data):
            print(data)

        c = BluetoothClient("raspberrypi", data_received)
        c.send("helloworld")

        pause()

    :param str server:
        The server name ("raspberrypi") or server MAC address
        ("11:11:11:11:11:11") to connect to. The server must be a paired device.

    :param data_received_callback:
        A function reference should be passed, this function will be called when
        data is received by the client.  The function should accept a single parameter
        which when called will hold the data received. Set to ``None`` if data
        received is not required.

    :param int port:
        The Bluetooth port the client should use, the default is 1.

    :param str device:
        The Bluetooth device to be used, the default is "hci0", if your device
        only has 1 Bluetooth adapter this shouldn't need to be changed.

    :param str encoding:
        The encoding standard to be used when sending and receiving byte data. The default is
        "utf-8".  If set to ``None`` no encoding is done and byte data types should be used.

    :param bool power_up_device:
        If ``True``, the Bluetooth device will be powered up (if required) when the
        server starts. The default is ``False``.

        Depending on how Bluetooth has been powered down, you may need to use :command:`rfkill`
        to unblock Bluetooth to give permission to Bluez to power on Bluetooth::

            sudo rfkill unblock bluetooth

    :param bool auto_connect:
        If ``True`` (the default), the Bluetooth client will automatically try
        to connect to the server at initialisation, if ``False``, the
        :meth:`connect` method will need to be called.

    """
    def __init__(self,
        server,
        data_received_callback,
        port = 1,
        device = "hci0",
        encoding = "utf-8",
        power_up_device = False,
        auto_connect = True):

        self._server = server
        self._data_received_callback = data_received_callback
        self._port = port
        self._power_up_device = power_up_device
        self._encoding = encoding

        self._setup_adapter(device)

        self._connected = False
        self._client_sock = None

        self._conn_thread = None

        if auto_connect:
            self.connect()

    @property
    def device(self):
        """
        The Bluetooth device the client is using. This defaults to "hci0".
        """
        return self.adapter.device

    @property
    def server(self):
        """
        The server name ("raspberrypi") or server `MAC address`_
        ("11:11:11:11:11:11") to connect to.

        .. _MAC address: https://en.wikipedia.org/wiki/MAC_address
        """
        return self._server

    @property
    def port(self):
        """
        The port the client is using. This defaults to 1.
        """
        return self._port

    @property
    def adapter(self):
        """
        A :class:`BluetoothAdapter` object which represents the Bluetooth
        device the client is using.
        """
        return self._adapter

    @property
    def encoding(self):
        """
        The encoding standard the client is using. The default is "utf-8".
        """
        return self._encoding

    @property
    def client_address(self):
        """
        The MAC address of the device being used.
        """
        return self.adapter.address

    @property
    def connected(self):
        """
        Returns ``True`` when connected.
        """
        return self._connected

    @property
    def data_received_callback(self):
        """
        Sets or returns the function which is called when data is received by the client.

        The function should accept a single parameter which when called will hold
        the data received. Set to ``None`` if data received is not required.
        """
        return self._data_received_callback

    @data_received_callback.setter
    def data_received_callback(self, value):
        self._data_received_callback = value

    def connect(self):
        """
        Connect to a Bluetooth server.
        """
        if not self._connected:

            if self._power_up_device:
                self.adapter.powered = True

            if not self.adapter.powered:
                raise Exception("Bluetooth device {} is turned off".format(self.adapter.device))

            #try and find the server name or MAC address in the paired devices list
            server_mac = None
            for device in self.adapter.paired_devices:
                if self._server == device[0] or self._server == device[1]:
                    server_mac = device[0]
                    break
            if server_mac == None:
                raise Exception("Server {} not found in paired devices".format(self._server))

            #create a socket
            self._client_sock = socket.socket(socket.AF_BLUETOOTH, socket.SOCK_STREAM, socket.BTPROTO_RFCOMM)
            self._client_sock.bind((self.adapter.address, self._port))
            self._client_sock.connect((server_mac, self._port))

            self._connected = True

            self._conn_thread = WrapThread(target=self._read)
            self._conn_thread.start()

    def disconnect(self):
        """
        Disconnect from a Bluetooth server.
        """
        if self._connected:

            #stop the connection thread
            if self._conn_thread:
                self._conn_thread.stop()
                self._conn_thread = None

            #close the socket
            try:
                self._client_sock.close()
            finally:
                self._client_sock = None
                self._connected = False

    def send(self, data):
        """
        Send data to a Bluetooth server.

        :param str data:
            The data to be sent.
        """
        if self._connected:
            if self._encoding is not None:
                data = data.encode(self._encoding)
            try:
                self._send_data(data)
            except IOError as e:
                self._handle_bt_error(e)

    def _send_data(self, data):
        """
        Send raw data to the client.
        
        :param bytes data:
            The data to be sent.
        """
        self._client_sock.send(data)

    def _read(self):
        #read until the client is stopped or the client disconnects
        while self._connected:
            #read data from Bluetooth socket
            try:
                data = self._client_sock.recv(1024, socket.MSG_DONTWAIT)
            except IOError as e:
                self._handle_bt_error(e)
                data = b""
            if data:
                #print("received [%s]" % data)
                if self._data_received_callback:
                    if self._encoding:
                        data = data.decode(self._encoding)
                    self.data_received_callback(data)
            if self._conn_thread.stopping.wait(BLUETOOTH_TIMEOUT):
                break

    def _setup_adapter(self, device):
        self._adapter = BluetoothAdapter(device)
    
    def _handle_bt_error(self, bt_error):
        assert isinstance(bt_error, IOError)
        #'resource unavailable' is when data cannot be read because there is nothing in the buffer
        if bt_error.errno == errno.EAGAIN:
            pass
        #'connection reset' is caused when the client disconnects
        elif bt_error.errno == errno.ECONNRESET:
            self._connected = False
        #'conection timeout' is caused when the server can no longer connect to read from the client
        # (perhaps the client has gone out of range)
        elif bt_error.errno == errno.ETIMEDOUT:
            self._connected = False
        else:
            raise bt_error



import sys
from setuptools import setup

if sys.version_info[0] == 2:
    if not sys.version_info >= (2, 7):
        raise ValueError('This package requires Python 2.7 or newer')
elif sys.version_info[0] == 3:
    if not sys.version_info >= (3, 3):
        raise ValueError('This package requires Python 3.3 or newer')
else:
    raise ValueError('Unrecognized major version of Python')

__project__ = 'bluedot'
__desc__ = 'A zero boiler plate bluetooth remote'
__version__ = '1.3.1'
__author__ = "Martin O'Hanlon"
__author_email__ = 'martin@ohanlonweb.com'
__license__ = 'MIT'
__url__ = 'https://github.com/martinohanlon/BlueDot'
__requires__ = ['dbus-python',]
__keywords__ = [
    "raspberry",
    "pi",
    "raspberry pi",
    "bluetooth",
    "remote",
]
__classifiers__ = [
#   "Development Status :: 3 - Alpha",
#   "Development Status :: 4 - Beta",
    "Development Status :: 5 - Production/Stable",
    "Intended Audience :: Education",
    "Intended Audience :: Developers",
    "Topic :: Education",
    "Topic :: Communications",
    "License :: OSI Approved :: MIT License",
    "Programming Language :: Python :: 2",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.3",
    "Programming Language :: Python :: 3.4",
    "Programming Language :: Python :: 3.5",
]
__long_description__ = """# Blue Dot

[Blue Dot](http://bluedot.readthedocs.io/en/latest/) allows you to control your Raspberry Pi projects wirelessly - it's a Bluetooth remote and zero boiler plate (super simple to use :) Python library.

## Getting started

1. Install

```
sudo pip3 install bluedot
```

2. Get the [Android Blue Dot app](http://play.google.com/store/apps/details?id=com.stuffaboutcode.bluedot) or use the [Python Blue Dot app](http://bluedot.readthedocs.io/en/latest/bluedotpythonapp.html)

3. Pair your Raspberry Pi

4. Write some code

```python
from bluedot import BlueDot
bd = BlueDot()
bd.wait_for_press()
print("You pressed the blue dot!")
```

5. Press the Blue Dot

See the [getting started guide](http://bluedot.readthedocs.io/en/latest/gettingstarted.html) to 'get statred'!

## More

The Blue Dot is a joystick as well as button. You can tell if the dot was pressed in the middle, on the top, bottom, left or right. You can easily create a BlueDot controlled Robot.

Why be restricted by such vague positions like top and bottom though: you can get the exact (x, y) position or even the angle and distance from centre where the dot was pressed.

Its not all about when the button was pressed either - pressed, released or moved they all work.

You can press it, slide it, swipe it, rotate it - one blue circle can do a lot!

## Even more

The [online documentation](http://bluedot.readthedocs.io/en/latest/) describes how to use Blue Dot and the Python library including recipes and ideas.

"""

if __name__ == '__main__':
    setup(name='bluedot',
          version = __version__,
          description = __desc__,
          long_description=__long_description__,
          long_description_content_type='text/markdown',
          url = __url__,
          author = __author__,
          author_email = __author_email__,
          license= __license__,
          keywords=__keywords__,
          classifiers=__classifiers__,
          packages = [__project__],
          #install_requires = __requires__,
          entry_points={
              'console_scripts': [
                  'bluedotapp = bluedot.app:main'
                  ]},
          zip_safe=False)
